#!/bin/bash

tests=`pwd`

export ROCKETGIT_CONF_FILE="`pwd`/config.php"
export ROCKETGIT_LOGIN_UID=1234
export ROCKETGIT_REPO_ID=5678
export ROCKETGIT_IP="127.0.0.1"
export ROCKETGIT_REPO_PATH="`pwd`/hook_update_dest.git"
export ROCKETGIT_REPO_UID=22

rm -rf hook_update_*.git
mkdir hook_update_dest.git
(
cd hook_update_dest.git
git init --bare
)
cp -v ../hooks/update hook_update_dest.git/hooks/

ln -sf ../../root/themes/default themes/

git clone hook_update_dest.git hook_update_src.git

cd hook_update_src.git

php ${tests}/hook_update_help.php init
if [ "${?}" != "0" ]; then
	echo "Cannot init user and repo!"
	exit 1
fi

echo
echo "=== Testing push without rights..."
echo "aaa" > a
git add a
git commit -m "a" a >/dev/null || exit 1
echo "bbb" >> a
git commit -m "b" a >/dev/null || exit 1
#strace -s200 -ff -o ${tests}/hook_update.strace \
git push origin master
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing push with rights..."
export ROCKETGIT_REPO_RIGHTS="CP"
php ${tests}/hook_update_help.php refs
#strace -s200 -ff -o ${tests}/hook_update.strace \
git push origin master
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing not fast-forward without rights..."
git reset --hard HEAD~1
echo "bbb" > a
git commit -m "new a" a
git push --force origin master
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing not fast-forward with rights..."
export ROCKETGIT_REPO_RIGHTS="COP"
php ${tests}/hook_update_help.php refs
git push --force origin master
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing un-annotated tag creation without rights..."
git tag tag1
git push --tags origin
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing un-annotated tag creation with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}Y"
php ${tests}/hook_update_help.php refs
git push --tags origin
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing un-annotated tag modify without rights..."
git tag -d tag1
git tag tag1 HEAD~1
git push --force --tags origin
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing un-annotated tag modify with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}U"
php ${tests}/hook_update_help.php refs
git push --force --tags origin
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing un-annotated tag deletion without rights..."
git tag -d tag1
git push origin :refs/tags/tag1
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing un-annotated tag deletion with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}u"
php ${tests}/hook_update_help.php refs
git push origin :refs/tags/tag1
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing merge-commit without rights (${ROCKETGIT_REPO_RIGHTS})..."
git checkout -b branch1
echo "ccc" >> a
git commit -m "new b" a
git checkout master
echo "a" > b; git add b
git commit -m "c" b
git merge -m "x" branch1
git push origin master
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing merge-commit with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}M"
php ${tests}/hook_update_help.php refs
git push origin master
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing bad-whitespace without rights..."
echo -e " \t \ta \t\n \r\n" > b
git commit -m "d" b
git push origin master
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing bad-whitespace with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}W"
php ${tests}/hook_update_help.php refs
git push origin master
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing branch delete without rights..."
git push origin branch1
git push origin :branch1
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing branch delete with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}D"
php ${tests}/hook_update_help.php refs
git push origin :branch1
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing annotated tag create without rights..."
git tag -a tag2 -m "xxx"
git push origin --tags
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing annotated tag create with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}S"
php ${tests}/hook_update_help.php refs
git push origin --tags
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Testing annotated tag delete without rights (${ROCKETGIT_REPO_RIGHTS})..."
git tag -d tag2
git push origin :refs/tags/tag2
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing annotated tag delete with rights..."
export ROCKETGIT_REPO_RIGHTS="${ROCKETGIT_REPO_RIGHTS}n"
php ${tests}/hook_update_help.php refs
git push origin :refs/tags/tag2
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi

echo
echo "=== Preparing for anonymous push..."
# We should allow only W? TODO
export ROCKETGIT_REPO_RIGHTS="W"
php ${tests}/hook_update_help.php refs
export GIT_NAMESPACE="hook_update_ns1"
echo "xx" > b; git add b
git commit -m "preparing for anonymous push" b
# We need to do this because it is done in remote.php that we do not call here
X="../hook_update_dest.git/refs/namespaces/${GIT_NAMESPACE}/refs/heads"
mkdir -p ${X}
cp -a ../hook_update_dest.git/refs/heads/* "${X}/"
echo "=== Testing anonymous push without rights..."
#strace -s200 -ff -o ${tests}/hook_update.strace \
git push origin master
if [ "${?}" != "1" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing anonymous push with rights..."
export ROCKETGIT_REPO_RIGHTS="HW"
php ${tests}/hook_update_help.php refs
git push origin master
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi
# Test if the namespace has been updated instead of main namespace
A=`diff ../hook_update_dest.git/refs/namespaces/${GIT_NAMESPACE}/refs/heads/master \
	.git/refs/heads/master`
if [ "${?}" != 0 ]; then
	echo "Cannot do the diff for anonymous push"
	exit 1
fi
if [ "${A}" != "" ]; then
	echo "We did not update the namespace!"
	exit 1
fi
A=`diff ../hook_update_dest.git/refs/heads/master \
	.git/refs/heads/master`
if [ "${A}" = "" ]; then
	echo "Seems we updated also main namespace!"
	exit 1
fi

# TODO: we should not allow force pushes on anonymous branches?


echo "=== Preparing for branch anonymous push without rights..."
echo "*** Creating a branch..."
git checkout -b branch_anon1
echo "=== Testing branch anonymous push without rights..."
export ROCKETGIT_REPO_RIGHTS=""
php ${tests}/hook_update_help.php refs
git push origin branch_anon1:branch_anon1
if [ "${?}" = "0" ]; then
	echo "Should not work!"
	exit 1
fi
echo "=== Testing branch anonymous push with rights..."
export ROCKETGIT_REPO_RIGHTS="H"
php ${tests}/hook_update_help.php refs
git push origin branch_anon1:branch_anon1
if [ "${?}" != "0" ]; then
	echo "Should work!"
	exit 1
fi
# Test if the namespace has been updated instead of the namespace
if [ "${?}" != 0 ]; then
	echo "Cannot do the diff for anonymous push"
	exit 1
fi
A=`diff ../hook_update_dest.git/refs/namespaces/${GIT_NAMESPACE}/refs/heads/branch_anon1 \
	.git/refs/heads/branch_anon1`
if [ "${A}" != "" ]; then
	echo "We did not update the namespace but main refs (anon+branch)!"
	exit 1
fi
A=`diff ../hook_update_dest.git/refs/heads/master \
	.git/refs/heads/branch_anon1`
if [ "${A}" = "" ]; then
	echo "Seems we updated also main refs (anon+branch)!"
	exit 1
fi


cd ..

rm -rf hook_update_*.git

echo "hook_pre-update: OK!";
